package com.leromro.web.controller.system;

import java.util.List;
import java.util.Set;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.leromro.core.service.IClientAccountService;
import com.leromro.core.service.IVolunteerAccountService;
import com.leromro.system.service.ISysUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import com.leromro.common.constant.Constants;
import com.leromro.common.core.domain.AjaxResult;
import com.leromro.common.core.domain.entity.SysMenu;
import com.leromro.common.core.domain.entity.SysUser;
import com.leromro.common.core.domain.model.LoginBody;
import com.leromro.common.core.domain.model.LoginUser;
import com.leromro.common.utils.SecurityUtils;
import com.leromro.framework.web.service.SysLoginService;
import com.leromro.framework.web.service.SysPermissionService;
import com.leromro.framework.web.service.TokenService;
import com.leromro.system.service.ISysMenuService;

/**
 * 登录验证
 * 
 * @author ruoyi
 */
@Slf4j
@Api(tags = "登录验证")
@RestController
public class SysLoginController
{
    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private IVolunteerAccountService volunteerAccountService;

    @Autowired
    private IClientAccountService clientAccountService;

    /**
     * 登录方法
     * 
     * @param loginBody 登录信息
     * @return 结果
     */
    @PostMapping("/login")
    public AjaxResult login(@RequestBody LoginBody loginBody)
    {
        String token = loginService.login(loginBody.getCode());
        AjaxResult ajax = AjaxResult.success();
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    @PostMapping("/loginWeb")
    public AjaxResult loginWeb(@RequestBody LoginBody loginBody)
    {
        log.info("传来post请求，接到的参数为：{}",loginBody);
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.login(loginBody.getUsername(), loginBody.getPassword(), loginBody.getCode(),
                loginBody.getUuid());
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * 获取用户信息
     * 
     * @return 用户信息
     */
    @GetMapping("getInfo")
    public AjaxResult getInfo()
    {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        SysUser user = loginUser.getUser();
        // 角色集合
        Set<String> roles = permissionService.getRolePermission(user);
        // 权限集合
        Set<String> permissions = permissionService.getMenuPermission(user);
        if (!loginUser.getPermissions().equals(permissions))
        {
            loginUser.setPermissions(permissions);
            tokenService.refreshToken(loginUser);
        }
        AjaxResult ajax = AjaxResult.success();
        ajax.put("user", user);
        ajax.put("roles", roles);
        ajax.put("permissions", permissions);
        return ajax;
    }

    /**
     * 获取路由信息
     * 
     * @return 路由信息
     */
    @GetMapping("getRouters")
    public AjaxResult getRouters()
    {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuTreeByUserId(userId);
        return AjaxResult.success(menuService.buildMenus(menus));
    }

    @Transactional(rollbackFor = Exception.class)
    @PutMapping("setUserOrWorker/{userOrWorker}")
    public AjaxResult setUserOrWorker(@PathVariable Integer userOrWorker){
        Long userId = SecurityUtils.getUserId();
        SysUser user = new SysUser();
        user.setUserId(userId);
        user.setUserOrWorker(userOrWorker);
        userService.updateUserProfile(user);
        //为对应角色创建对应账号
        if (userOrWorker == 1){
            clientAccountService.insertVolunteerAccount(userId);
        }else if (userOrWorker == 2){
            volunteerAccountService.insertVolunteerAccount(userId);
        }
        return AjaxResult.success();
    }

    /**
     * 修改当前用户定位
     */
    @ApiOperation("修改当前用户定位")
    @PostMapping("/setUserLocation")
    public AjaxResult setUserOrWorker(@RequestBody SysUser user){
        Long userId = SecurityUtils.getUserId();
        user.setUserId(userId);
        userService.updateById(user);
        return AjaxResult.success();
    }
}
