package com.leromro.core.service.impl;

import java.math.BigDecimal;
import java.util.List;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.leromro.core.domain.vo.AccountChangeTotalMoney;
import com.leromro.core.domain.vo.ClientAccountChangeVO;
import com.leromro.core.domain.vo.VolunteerAccountChangeVO;
import com.leromro.core.mapper.VolunteerAccountChangeMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.leromro.core.domain.VolunteerAccountChange;
import com.leromro.core.service.IVolunteerAccountChangeService;

/**
 * 志愿者账号变动明细Service业务层处理
 * 
 * @author ruoyi
 * @date 2025-04-11
 */
@Service
public class VolunteerAccountChangeServiceImpl extends ServiceImpl<VolunteerAccountChangeMapper, VolunteerAccountChange> implements IVolunteerAccountChangeService
{
    @Autowired
    private VolunteerAccountChangeMapper volunteerAccountChangeMapper;

    /**
     * 查询志愿者账号变动明细
     * 
     * @param volunteerAccountChangeId 志愿者账号变动明细主键
     * @return 志愿者账号变动明细
     */
    @Override
    public VolunteerAccountChange selectVolunteerAccountChangeByVolunteerAccountChangeId(Long volunteerAccountChangeId)
    {
        return volunteerAccountChangeMapper.selectVolunteerAccountChangeByVolunteerAccountChangeId(volunteerAccountChangeId);
    }

    /**
     * 查询志愿者账号变动明细列表
     * 
     * @param volunteerAccountChange 志愿者账号变动明细
     * @return 志愿者账号变动明细
     */
    @Override
    public List<VolunteerAccountChange> selectVolunteerAccountChangeList(VolunteerAccountChange volunteerAccountChange)
    {
        return volunteerAccountChangeMapper.selectVolunteerAccountChangeList(volunteerAccountChange);
    }

    /**
     * 新增志愿者账号变动明细
     * 
     * @param volunteerAccountChange 志愿者账号变动明细
     * @return 结果
     */
    @Override
    public Boolean insertVolunteerAccountChange(VolunteerAccountChange volunteerAccountChange)
    {
        return this.save(volunteerAccountChange);
    }

    /**
     * 修改志愿者账号变动明细
     * 
     * @param volunteerAccountChange 志愿者账号变动明细
     * @return 结果
     */
    @Override
    public Boolean updateVolunteerAccountChange(VolunteerAccountChange volunteerAccountChange)
    {
        return this.updateById(volunteerAccountChange);
    }

    /**
     * 批量删除志愿者账号变动明细
     * 
     * @param volunteerAccountChangeIds 需要删除的志愿者账号变动明细主键
     * @return 结果
     */
    @Override
    public int deleteVolunteerAccountChangeByVolunteerAccountChangeIds(Long[] volunteerAccountChangeIds)
    {
        return volunteerAccountChangeMapper.deleteVolunteerAccountChangeByVolunteerAccountChangeIds(volunteerAccountChangeIds);
    }

    /**
     * 删除志愿者账号变动明细信息
     * 
     * @param volunteerAccountChangeId 志愿者账号变动明细主键
     * @return 结果
     */
    @Override
    public int deleteVolunteerAccountChangeByVolunteerAccountChangeId(Long volunteerAccountChangeId)
    {
        return volunteerAccountChangeMapper.deleteVolunteerAccountChangeByVolunteerAccountChangeId(volunteerAccountChangeId);
    }

    /**
     * @return
     */
    @Override
    public AccountChangeTotalMoney getVolunteerAccountTotalChangeMoney(VolunteerAccountChange volunteerAccountChange) {
        AccountChangeTotalMoney totalMoney = new AccountChangeTotalMoney();
        List<VolunteerAccountChangeVO> list = volunteerAccountChangeMapper.selectVolunteerAccountChangeListAndServiceType(volunteerAccountChange);
        //算出总收入以及总支出
        BigDecimal totalEarning = list.stream()
                .filter(vo -> "1".equals(vo.getChangeType()))
                .map(VolunteerAccountChangeVO::getChangeMoney)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal totalExpend = list.stream()
                .filter(vo -> "0".equals(vo.getChangeType()))
                .map(VolunteerAccountChangeVO::getChangeMoney)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        totalMoney.setTotalExpend(totalExpend);
        totalMoney.setTotalEarning(totalEarning);
        return totalMoney;
    }

    /**
     * @param volunteerAccountChange
     * @return
     */
    @Override
    public List<VolunteerAccountChangeVO> selectVolunteerAccountChangeListInfo(VolunteerAccountChange volunteerAccountChange) {
         List<VolunteerAccountChangeVO> volunteerAccountChangeListVO =volunteerAccountChangeMapper.selectVolunteerAccountChangeListAndServiceType(volunteerAccountChange);
        return volunteerAccountChangeListVO;
    }

}
